----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:27:27 12/10/2008 
-- Design Name: 
-- Module Name:    Adress - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use WORK.CPU_PACKAGE.ALL;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity AdressInterface is
    Port (  Reset:in STD_LOGIC ;
		    Clk : in  STD_LOGIC;
			  
            AdressBusIn : in  Adress ; -- Adresseingang, der von Control kommt
		    AdressBusOut: out Adress ; -- Adressausgang an die Speichereinheit
			  
			DataToMemory: out Data ;  
			DataFromMemory: in Data ;
			DToControl : out  Data ;  -- Daten zur Control
			  
            PCOut : in  Data ;  -- Daten von der Control
			AccOut: in Data ;   -- Daten von der ALU
			  
            MemEn : in  STD_LOGIC;
            RW : in  STD_LOGIC;	-- R/W von der Control
			RWOut :out STD_LOGIC ; -- R/W zum Speicher
			  
			SelectSource: in STD_LOGIC -- Steuersignal zum multiplexen von PCOut und AccOut
			);
end AdressInterface;

architecture Behavioral of AdressInterface is
	signal AddrReg : Adress := (others => '0') ;
begin
	
	-- durchschalten von RW
	RW_through:process(RW)
	begin
		RWOut <= RW ;
	end process ;
	
	-- durchschalten von DataFromMemory
	Data_through:process(DataFromMemory)
	begin
		DToControl <= DataFromMemory ;
	end process ;

	-- multiplexen von daten(PC von Control, AKK von ALU) zur speichereinheit
	write_datareg: process(MemEn, RW )
	begin
		if (MemEn = '1') then
			if (RW = '1') then
				if (SelectSource = '0') then
					DataToMemory <= PCOut ;
				else
					DataToMemory <= AccOut ;
				end if ;
			end if ;
		end if ;
	end process ;
	
	-- schreibe addresse an speichereinheit
	-- Falls Schreibvorgang: adresseinheit inkrementiert adresse
	-- Falls Lesevorgang an der ALU: Adresseinheit inkrementiert A., sonst Control
	write_addr_out:process(Clk, Reset, MemEn, SelectSource, AdressBusIn)
		variable ct : integer := 0 ;
	begin
		if (Reset = '1') then
			ct := 0 ;
			AdressBusOut <= (others => '0') ;
			AddrReg <= (others => '0') ;
			
		elsif (Clk = '1' and Clk'event) then
			if (MemEn = '1') then
				if (ct = 0) then
					if (RW = '1' or (SelectSource = '1' and RW = '0')) then 
						ct := ct + 1 ;  
					end if ;
						
					AddrReg 			<= AdressBusIn ;
					AdressBusOut 	<= AdressBusIn ;
				else
					AdressBusOut <= AddrReg + 1 ;
					ct := 0 ;
				end if ;
			end if ;
		end if ;			
	end process ;
		
end architecture ;
